@@ -89,12 +89,35 @@ int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 int ssl_secure_flag=0;
-int ssl_certsok_flag=0;
+int ssl_certsok_flag=1;
+int ssl_cert_required=0;
+int ssl_verbose_flag=0;
+int ssl_disabled_flag=0;
 char *cert_file=NULL;
 char *key_file=NULL;
 char *cipher_list=NULL;
 
 extern int display_connect_details();
+extern int server_verify_callback();
+extern int client_verify_callback();
+
+static void client_info_callback(s,where,ret)
+SSL *s;
+int where;
+int ret;
+{
+	if (where == SSL_CB_CONNECT_LOOP) {
+		fprintf(stderr,"SSL_connect:%s %s\n",SSL_state_string(s),SSL_state_string_long(s));
+	} else if (where == SSL_CB_CONNECT_EXIT) {
+		if (ret == 0) {
+			fprintf(stderr,"SSL_connect:failed in %s %s\n",
+				SSL_state_string(s),SSL_state_string_long(s));
+		} else if (ret < 0) {
+			fprintf(stderr,"SSL_connect:error in %s %s\n",
+				SSL_state_string(s),SSL_state_string_long(s));
+		}
+	}
+}
 
 #endif /* USE_SSL */
 
@@ -2293,9 +2316,30 @@ tn(argc, argv)
 	    if (sp)
 		sin.sin_port = sp->s_port;
 	    else {
+#ifdef USE_SSL
+		/* hardwire in the knowledge of https as I'm
+		 * getting tierd of machines that are configured
+		 * without knowledge of this! --tjh
+		 *
+		 * ... actually I've extended this to include
+		 * all the usual ports that I like going to that
+		 * various machine administrator have been too
+		 * slack to setup
+		 *
+		 */
+		if (strcmp(portp,"http")==0)
+		    sin.sin_port = 80;
+		else if (strcmp(portp,"https")==0)
+		    sin.sin_port = 443;
+		else if (strcmp(portp,"ssl-telnet")==0)
+		    sin.sin_port = 151;
+		else
+#endif /* USE_SSL */
+                {
 		printf("%s: bad port number\n", portp);
 	        setuid(getuid());
 		return 0;
+		}
 	    }
 	} else {
 #if	!defined(htons)
@@ -2406,12 +2450,19 @@ tn(argc, argv)
 	else
 	    SSL_set_pref_cipher(ssl_con,cipher_list);
 
+	/* for verbose we use the 0.6.x info callback that I got
+	 * eric to finally add into the code :-) --tjh
+	 */
+	if (ssl_verbose_flag) {
+	    SSL_CTX_set_info_callback(ssl_ctx,client_info_callback);
+	}
+
 	/* Add in any certificates if you want to here ... */
 	if (cert_file) {
 	    if (!SSL_use_certificate_file(ssl_con, cert_file,
 		X509_FILETYPE_PEM)) {
 		fprintf(stderr,"%s: ",cert_file);
-		ERR_print_errors(stderr);
+		ERR_print_errors_fp(stderr);
 		exit(1);
 	    } else {
 		if (!key_file)
@@ -2419,21 +2470,30 @@ tn(argc, argv)
 		if (!SSL_use_RSAPrivateKey_file(ssl_con, key_file,
 		    X509_FILETYPE_PEM)) {
 		    fprintf(stderr,"%s: ", key_file);
-		    ERR_print_errors(stderr);
+		    ERR_print_errors_fp(stderr);
 		    exit(1);
 		}
 	    }
 	}
 
+	/* make sure we will find certificates in the standard
+	 * location ... otherwise we don't look anywhere for
+	 * these things which is going to make client certificate
+	 * exchange rather useless :-)
+	 */
+	SSL_set_default_verify_paths(ssl_ctx);
+
 	if (ssl_only_flag) {
+	    SSL_set_verify(ssl_con,ssl_verify_flag,
+					client_verify_callback);
 	    if (!SSL_connect(ssl_con)) {
                 static char errbuf[1024];
 
-		ERR_print_errors(stderr);
+		ERR_print_errors_fp(stderr);
 		perror("SSL_connect");
 		fflush(stderr);
 
-		return 0;
+		exit(1);
 	    } else {
 		display_connect_details(ssl_con,1);
 		ssl_active_flag=1;
