@@ -54,8 +54,9 @@
 #include <sys/filio.h>
 #endif
 #endif
+
 #ifdef	USE_TERMIO
-# ifndef	VINTR
+# if !defined(VINTR) || defined(LINUX)
 #  ifdef SYSV_TERMIO
 #   include <sys/termio.h>
 #  else
@@ -156,6 +157,7 @@ extern char
     wont[],
     options[],		/* All the little options */
     *hostname;		/* Who are we connected to? */
+
 #if	defined(ENCRYPT)
 extern void (*encrypt_output) P((unsigned char *, int));
 extern int (*decrypt_input) P((int));
@@ -468,16 +470,42 @@ extern void
     inputAvailable P((void));
 #endif	/* defined(TN3270) */
 
+/* protect ENCRYPT ... des.h defines it until I 
+ * can hit Eric over the head with a *large* book --tjh
+ */
+#undef OLDENCRYPT
+#ifdef ENCRYPT
+#define OLDENCRYPT ENCRYPT
+#endif
+
 #ifdef USE_SSL
+#include "crypto.h"
+#include "x509.h"
 #include "ssl.h"
+
 extern SSL *ssl_con;
 extern int ssl_debug_flag;
 extern int ssl_only_flag;
 extern int ssl_active_flag;
 extern int ssl_verify_flag;
 extern int ssl_secure_flag;
+extern int ssl_verbose_flag;
+extern int ssl_disabled_flag;
+extern int ssl_cert_required;
+
+extern char *cert_file; 
+extern char *key_file;
+extern char *cipher_list;
 #endif /* USE_SSL */
 
 #ifdef USE_SOCKS
 extern int socks_flag;
 #endif /* USE_SOCKS */
+
+#ifdef OLDENCRYPT
+#define ENCRYPT OLDENCRYPT
+#else
+#undef ENCRYPT
+#endif
+
+
