@@ -80,7 +80,7 @@ char	wtmpf[]	= "/etc/wtmp";
 #include <sys/stream.h>
 #include <sys/stropts.h>
 #endif
-#if !(defined(LINUX) || defined(__sgi) || defined(HPUX))
+#if !(defined(LINUX) || defined(__sgi) || defined(HPUX) || defined(__bsdi__))
 #include <sys/tty.h>
 #endif
 #ifdef	t_erase
