@@ -105,9 +105,13 @@ int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_secure_flag=0;
 int ssl_certsok_flag=0;
+int ssl_cert_required=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 char ssl_file_path[1024];    /* don't look at that nasty value to the left */
 
+int ssl_verbose_flag=0;
+int ssl_disabled_flag=0;
+
 static char *my_ssl_key_file=NULL;
 static char *my_ssl_cert_file=NULL;
 char *cipher_list;
@@ -152,31 +156,62 @@ main(argc, argv)
 
 #ifdef USE_SSL
                 case 'z':
+		        { 
+			char *origopt;
+
+			origopt=strdup(optarg);
+			optarg=strtok(origopt,",");
+
+			while(optarg!=NULL) {
+
 		        if (strcmp(optarg, "debug") == 0 ) {
 			    ssl_debug_flag=1;
-			}
-		        if (strcmp(optarg, "ssl") == 0 ) {
+			} else if (strcmp(optarg, "ssl") == 0 ) {
 			    ssl_only_flag=1;
-			}
-		        if (strcmp(optarg, "certsok") == 0 ) {
+			} else if (strcmp(optarg, "certsok") == 0 ) {
 			    ssl_certsok_flag=1;
-			}
-		        if (strcmp(optarg, "secure") == 0 ) {
+			} else if ( (strcmp(optarg, "!ssl") == 0) ||
+		             (strcmp(optarg, "nossl") == 0) ) {
+			    /* we may want to switch SSL negotiation off
+			     * for testing or other reasons 
+			     */
+			    ssl_disabled_flag=1;
+			} else if (strcmp(optarg, "certrequired") == 0 ) {
+			    ssl_cert_required=1;
+			} else if (strcmp(optarg, "secure") == 0 ) {
 			    ssl_secure_flag=1;
-			}
-		        if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
+			} else if (strncmp(optarg, "verify=", 
+			                strlen("verify=")) == 0 ) {
 			    ssl_verify_flag=atoi(optarg+strlen("verify="));
-
-			}
-		        if (strncmp(optarg, "cert=", strlen("cert=")) == 0 ) {
+			} else if (strncmp(optarg, "cert=", 
+			                strlen("cert=")) == 0 ) {
 			    my_ssl_cert_file=optarg+strlen("cert=");
-			}
-		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
+			} else if (strncmp(optarg, "key=", 
+			                strlen("key=")) == 0 ) {
 			    my_ssl_key_file=optarg+strlen("key=");
+			} else if (strncmp(optarg,"cipher=",
+			                strlen("cipher="))==0) {
+			    cipher_list=optarg+strlen("cipher=");
+			} else {
+			    /* report when we are given rubbish so that
+			     * if the user makes a mistake they have to
+			     * correct it!
+			     */
+			    fprintf(stderr,"Unknown SSL option %s\n",optarg);
+			    fflush(stderr);
+			    exit(1);
 			}
 
-		        if (strncmp(optarg,"cipher=",sizeof("cipher"))==0) {
-			    cipher_list=optarg+sizeof("cipher");
+			/* get the next one ... */
+                        optarg=strtok(NULL,",");
+
+			}
+
+			/*
+			if (origopt!=NULL)
+			    free(origopt);
+			*/
+
 			}
 			break;
 #endif /* USE_SSL */
@@ -348,6 +383,16 @@ main(argc, argv)
 	}
 
 #ifdef USE_SSL
+
+        if (ssl_secure_flag || ssl_cert_required) {
+	    /* in secure mode we *must* switch on the base level
+	     * verify checking otherwise we cannot abort connections
+	     * at the right place!
+	     */
+	    if (ssl_verify_flag == 0)
+		ssl_verify_flag = 1;
+	}
+
         {
 	    int i;
 
@@ -358,22 +403,36 @@ main(argc, argv)
 	     */
 	    if (ssl_debug_flag) {
 		if (!debug) {
-		    (void)freopen("telnetd.err","w",stderr);
+		    (void)freopen("/telnetd.err","w",stderr);
 		    SSL_ERR=stderr;
 		}
-		SSL_debug("telnetd.log");
+		SSL_debug("/telnetd.log");
+
+		fprintf(stderr,"LOGGING to telnetd.err and telnetd.log\n");
+
+		fprintf(stderr,"secure %d certrequired %d verify %d\n",
+		    ssl_secure_flag,ssl_cert_required,ssl_verify_flag);
+		for(i=0;i<argc;i++)
+		    fprintf(stderr,"argv[%d]=\"%s\"\n",i,argv[i]);
+		fflush(stderr);
+
 	    } else {
 		/* disable all the debug and trace */
 		SSL_LOG=SSL_ERR=NULL;
 	    }
+
 	    SSL_load_error_strings();
 
 	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
 
-            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
-		fprintf(stderr,"telnetd: cannot load certificates via X509_set_default_verify_paths\n");
+	    /* make sure we have access to the certificate area 
+	     * otherwise we will not be able to verify any client
+	     * certificates
+	     */
+            if (!SSL_set_default_verify_paths(ssl_ctx)) {
+		fprintf(stderr,"telnetd: cannot load certificates via SSL_set_default_verify_paths\n");
 		fflush(stderr);
-		ERR_print_errors(stderr);
+		ERR_print_errors_fp(stderr);
 		exit(1);
 	    }
 
@@ -495,7 +554,7 @@ main(argc, argv)
 		X509_FILETYPE_PEM)<=0) {
 	    fprintf(stderr,"SSL_use_RSAPrivateKey_file(%s) error ",
 	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file);
-	    ERR_print_errors(stderr);
+	    ERR_print_errors_fp(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
@@ -508,7 +567,7 @@ main(argc, argv)
 	        X509_FILETYPE_PEM)<=0) {
 	    fprintf(stderr,"SSL_use_certificate_file(%s) error ",
 	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file);
-	    ERR_print_errors(stderr);
+	    ERR_print_errors_fp(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
@@ -634,7 +693,6 @@ getterminaltype(name)
 	     * to do this cleanly at this point!
 	     */
 
-
 #if 0
             /* this muck is needed so that the message
 	     * actually makes it back to the user ...
@@ -656,6 +714,11 @@ getterminaltype(name)
 	    }
 #endif
 
+            if (ssl_debug_flag) {
+		fprintf(stderr,"[SSL required - connection rejected]");
+		fflush(stderr);
+	    }
+
 	    fatal(net,"[SSL required - connection rejected]");
 
 	}
